# -*- coding: UTF-8 -*-
# Copyright 2014 Red Hat, Inc.
# Part of clufter project
# Licensed under GPLv2+ (a copy included | http://gnu.org/licenses/gpl-2.0.txt)

ccs2ccs_pcmk = '''\
    <fencedevices>
        <fencedevice agent="fence_pcmk"
                     name="pcmk-redirect"/>
    </fencedevices>
'''
