# -*- coding: UTF-8 -*-
# Copyright 2014 Red Hat, Inc.
# Part of clufter project
# Licensed under GPLv2+ (a copy included | http://gnu.org/licenses/gpl-2.0.txt)

from ....filters.ccs_artefacts import artefact

ccs_artefacts = artefact('@program', kind='C', desc='quorumd heuristic program')
